@@ -23,6 +23,7 @@ use apexl\Io\modules\subscription\services\subscriptionService;
 use apexl\Io\modules\user\entities\userEntity;
 use apexl\Io\services\Logger;
 use apexl\Io\services\Mailer;
+use apexl\Vault\Vault;
 use Psr\Http\Message\ResponseInterface as Response;
 use Psr\Http\Message\ServerRequestInterface as Request;
 
@@ -226,15 +227,16 @@ class invoiceService {
     public function createInvoiceLineItem($description, $quantity, $perUnitCost, $nominalCode, $productId=null, $taxType=null)
     {
         if ($quantity == 0) return [];
-        $total = number_format($quantity*$perUnitCost, 2);
+        $total = number_format($quantity*$perUnitCost, 2, '.', '');
+        $totalRaw = round($quantity*$perUnitCost, 2);
         //$nominalCode = '200';
         $lineItem = (object)[
             'description' => $description,
             'accountCode' => $nominalCode,
             'quantity' => $quantity,
-            'perUnit' => $perUnitCost,
-            'amount' => !in_array('ECZROUTPUT', 'ZERORATEDOUTPUT') ? $total : round(1.2*$total, 2),
-            'vat' => !in_array($taxType, 'NONE', 'ECZROUTPUT', 'ZERORATEDOUTPUT') ? number_format(0.2*$total, 2) : 0,
+            'perUnit' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $total : number_format(1.2*$totalRaw, 2, '.', ''),
+            'amount' => !in_array($taxType, ['ECZROUTPUT', 'ZERORATEDOUTPUT']) ? $total : number_format(1.2*$totalRaw, 2, '.', ''),
+            'vat' => !in_array($taxType, ['NONE', 'ECZROUTPUT', 'ZERORATEDOUTPUT']) ? number_format(0.2*$totalRaw, 2, '.', '') : 0,
             'productId' => $productId
         ];
         if ($taxType) {
@@ -295,12 +297,14 @@ class invoiceService {
      */
     public function triggerSubscriptionInvoices(Request $request, Response $response, $args)
     {
+        if (date('H') < 8 || date('H') > 17) return System::asJson($response, ['subscriptionsProcessed' => 0, 'reason' => 'before 8am or after 6pm']);
+
         if (!self::checkStartCron()) return System::asJson($response, ['error' => 'exit due to cron duplication. process: '.getmypid()]);
 
         $subscriptionEntity = new subscriptionEntity();
         $productEntity = new productEntity();
         $subscriptionService = new subscriptionService();
-        $toBill = $subscriptionEntity->getSubscriptionsToBill(date('Y-m-d'), 4, 24);
+        $toBill = $subscriptionEntity->getSubscriptionsToBill(date('Y-m-d'), 1, 24);
         $subscriptionsProcessed = 0;
 
         foreach ($toBill as $userId => $providerSubscriptions) {
@@ -346,7 +350,10 @@ class invoiceService {
                 $subscriptionsProcessed++;
                 $allLineItems = array_merge($allLineItems, $lineItems);
             }
-
+                //echo '<pre>';
+                //print_r($allLineItems);
+                //echo '</pre>';
+                //die();
             if (count($allLineItems) > 0) { // Create the invoice
                 if (!self::checkContinueCron()) return System::asJson($response, ['error' => 'exit due to cron duplication. process: '.getmypid()]);
                     Hook::processHook('pre_raise_subscription_invoice', $subscriptions);
@@ -414,6 +421,61 @@ class invoiceService {
         return $result;
     }
 
+    public function triggerPaymentAlerts(Request $request, Response $response, $args)
+    {
+
+        $checks = [
+            (object)[
+                'varName' => 'cron_subscription_billing_completed',
+                'triggerDays' => '2',
+                'alert' => 'Cron Subscription Billing Invoice Generation failed to complete for more than two daya'
+            ],
+            (object)[
+                'varName' => 'cron_invoice_payments_completed',
+                'triggerDays' => '2',
+                'alert' => 'Cron Invoice Charging failed to complete for more than two daya'
+            ]
+        ];
+
+        $alerts = [];
+        foreach ($checks as $check) {
+            $lastSuccessfulRun = System::getVariable($check->varName);
+            if ($lastSuccessfulRun) {
+                $lastSuccessfulRun = \DateTime::createFromFormat('U', $lastSuccessfulRun);
+                $lastSuccessfulRun->modify("+{$check->triggerDays} days");
+                $now = new \DateTime('now');
+                if ($lastSuccessfulRun < $now) {
+                    $alerts[] = $check->alert;
+                }
+            }
+        }
+
+        $lastAlertEmail = System::getVariable('payment_alert_email_sent');
+        if ($lastAlertEmail) {
+            $lastSent = \DateTime::createFromFormat('U', $lastAlertEmail);
+        }
+        if (count($alerts) > 0 && (!$lastAlertEmail || ((int)$lastAlertEmail+84600) < time())) {
+            $emailData = [
+                'subject' => 'URGENT: GlobalM2MSIM Dashboard Cron failures',
+                'message' => '<p>'.implode("<br />", $alerts).'</p>',
+            ];
+            $from = $this->config->app->site->email_address ?? 'no-reply@localhost.com';
+            $fromName = $this->config->app->site->name ?? 'localhost';
+            $mailer = new Mailer();
+            $mailer->setFrom($from, $fromName);
+            $mailer->addAddress("mike@developmentx.uk");     // Add a recipient
+
+            $mailer->Subject = 'URGENT: GlobalM2MSIM Dashboard Cron failures';
+            $mailer->Body = templateService::fetch("generic", $emailData);
+            $mailer->IsHTML(true);
+            $result = $mailer->send();
+
+            System::setVariable('payment_alert_email_sent', date('U'));
+
+            Logger::log('payment_alert_email_sent', ['sent' => $result, 'alerts' => $alerts]);
+        }
+    }
+
     protected static function checkStartCron($process='subscription')
     {
         $lastStarted = System::getVariable($process.'_cron_process_started');
@@ -467,6 +529,42 @@ class invoiceService {
         }
 
         // 2. Check balance in Xero
+        if ($this->downloadProviderPaidStatus($invoiceEntity)) return true;
+
+        return false;
+
+    }
+
+    public function backfillProviderPaidStatus()
+    {
+        $vault = Vault::getInstance();
+        $invoice = $vault
+            ->select('invoice')
+            ->fields()
+            ->where('paid', 0)
+            ->where('payment_provider', 'gocardless')
+            ->where('provider_id', NULL, 'IS NOT NULL')
+            ->where('paid_status_check', NULL, 'IS NULL')
+            ->orderBy('invoice_date', 'DESC')
+            ->limit(1)
+            ->execute()
+            ->fetchAssoc();
+
+        if (isset($invoice['provider_id'])) {
+            $invoiceEntity = new invoiceEntity();
+            $invoiceEntity->load($invoice['id']);
+            try {
+                $this->downloadProviderPaidStatus($invoiceEntity);
+            } catch (\Exception $e) {
+                $invoiceEntity->paid_status_check_error = $e->getMessage();
+            }
+            $invoiceEntity->paid_status_check = date('Y-m-d H:i:s');
+            $invoiceEntity->store();
+        }
+    }
+
+    protected function downloadProviderPaidStatus($invoiceEntity)
+    {
         if (isset($invoiceEntity->provider_id) && $invoiceEntity->provider_id) { // Submit payment information to provider
             $invoice = $this->provider->getInvoice($invoiceEntity->provider_id);
             if ($invoice->AmountDue < 0.015) {
